@@ -291,11 +291,34 @@ class SharesTest(utils.TestCase):
         search_opts = {
             'fake_str': 'fake_str_value',
             'fake_int': 1,
+            'name~': 'fake_name',
+            'description~': 'fake_description',
         }
         cs.shares.list(detailed=False, search_opts=search_opts)
         cs.assert_called(
             'GET',
-            '/shares?fake_int=1&fake_str=fake_str_value&is_public=True')
+            '/shares?description%7E=fake_description&fake_int=1&'
+            'fake_str=fake_str_value&is_public=True&name%7E=fake_name')
+
+    @ddt.data(('id', 'b4991315-eb7d-43ec-979e-5715d4399827', True),
+              ('id', 'b4991315-eb7d-43ec-979e-5715d4399827', False),
+              ('path', 'fake_path', False),
+              ('path', 'fake_path', True))
+    @ddt.unpack
+    def test_list_shares_index_with_export_location(self, filter_type,
+                                                    value, detailed):
+        search_opts = {
+            'export_location': value,
+        }
+        cs.shares.list(detailed=detailed, search_opts=search_opts)
+        if detailed:
+            cs.assert_called(
+                'GET', ('/shares/detail?export_location_' + filter_type +
+                        '=' + value + '&is_public=True'))
+        else:
+            cs.assert_called(
+                'GET', ('/shares?export_location_' + filter_type + '='
+                        + value + '&is_public=True'))
 
     def test_list_shares_detailed(self):
         cs.shares.list(detailed=True)
@@ -366,6 +389,18 @@ class SharesTest(utils.TestCase):
          'action_name': 'allow_access', 'microversion': '2.13'},
         {'access_to': 'alice bob', 'access_type': 'cephx',
          'action_name': 'allow_access', 'microversion': '2.13'},
+        {'access_to': 'AD80:0000:0000:0000:ABAA:0000:00C2:0002',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'AD80::/36',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'AD80:ABAA::/128',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
+        {'access_to': 'ad80::abaa:0:c2:2',
+         'access_type': 'ip', 'action_name': 'allow_access',
+         'microversion': '2.38'},
     )
     @ddt.unpack
     def test_allow_access_to_share(self, access_to, access_type,
@@ -416,6 +451,12 @@ class SharesTest(utils.TestCase):
          'microversion': '2.13'},
         {'access_to': u"bj\u00F6rn", 'access_type': 'cephx',
          'microversion': '2.13'},
+        {'access_to': "AD80:0000:0000:0000:ABAA:0000:00C2:0002/65",
+         'access_type': 'ip', 'microversion': '2.38'},
+        {'access_to': "AD80:0000:0000:0000:ABAA:0000:00C2:0002*32",
+         'access_type': 'ip', 'microversion': '2.38'},
+        {'access_to': "AD80:0000:0000:0000:ABAA:0000:00C2:0002",
+         'access_type': 'ip', 'microversion': '2.37'},
     )
     @ddt.unpack
     def test_allow_access_to_share_error_invalid_access(self, access_to,
